from xdict.prepare_quotes_token_machine  import prepare_quotes_token_machine


def tokenize_quotes(machine):
    curr_state = "INIT"
    rslt = ''
    for i in range(0,machine.orig_str.__len__()):
        input_symbol = machine.orig_str[i]
        action,next_state,trigger_checker = machine.search(curr_state,input_symbol)
        #print('----------')
        #print(curr_state,trigger_checker,input_symbol,action,next_state)
        if(action == machine.do_replace):
            ch = action(input_symbol)
            #print(ch)
        elif(action == machine.do_throw):
            ch = ''
            action(curr_state,trigger_checker,input_symbol)
        else:
            ch = input_symbol
        #rslt = ''.join((rslt,ch))
        rslt = rslt +ch
        curr_state = next_state
    return(rslt)



#######################
def convert_token_in_quote(j_str,**kwargs):
    '''
        #1. use html.escape  to  escape all quote-operators appeared in single-quote or double-quote
        #2. use html.escape  to  escape all paired-operators appeared in single-quote or double-quote : such as  "{,},[,],(,)"
        #3. use html.escape  to  escape all other operators appeared in single-quote or double-quote : such as : ':',','
        #4. use html_escape  to  escape seperators-operators appeared in single-quote or double-quote: such as '\n'

        >>> from xdict.jprint import convert_token_in_quote
        >>> from xdict.jprint import help
        >>>
        >>> convert_token_in_quote('"a b":"cd"')
        '"a&#32;b":"cd"'
        >>> import html
        >>> html.unescape('"a&#32;b":"cd"')
        '"a b":"cd"'
        >>> convert_token_in_quote('"a b":cd')
        '"a&#32;b":cd'
        >>>
        >>> #help(convert_token_in_quote)
        convert_token_in_quote('<a b>:"cd"',quotes_pairs_dict={1: ('"', '"'), 2: ("<", ">")})
        '<a&#32;b>:"cd"'
    '''
    machine = prepare_quotes_token_machine(j_str,**kwargs)
    rslt = tokenize_quotes(machine)
    return(rslt)




from xdict import block


def get_j_str_lvs_dict(j_str,block_op_pairs_dict=block.get_block_op_pairs("{}[]()")):
    j_str_len = j_str.__len__()
    j_str_lvs_dict = {}
    if( j_str_len == 0):
        j_str_lvs_dict = {}
    elif(j_str_len == 1):
        j_str_lvs_dict = {0:1}
    else:
        curr_lv = 1
        j_str_lvs_dict = {0:1}
        seq = 1
        curr_seq = 0
        while(curr_seq < j_str_len - 1):
            level,curr_lv,curr_seq = get_next_char_level_in_j_str(curr_lv,curr_seq,j_str,block_op_pairs_dict)
            j_str_lvs_dict[seq] =level
            seq = seq + 1
    return(j_str_lvs_dict)


def get_next_char_level_in_j_str(curr_lv,curr_seq,j_str,block_op_pairs_dict=block.get_block_op_pairs("{}[]()")):
    ''' the first-char is level-1
        when current is  non-op, next-char-level = curr-level
        when current is  lop,  non-paired-rop-next-char-level = lop-level+1;
        when current is  lop,  paired-rop-next-char-level = lop-level
        when current is  rop,  next-char-level = rop-level - 1
        # {"key_4_UF0aJJ6v": "value_1", "key_2_Hd0t": ["value_16", "value_8", "value_8", "value_15", "value_14", "value_19", {......
        # 122222222222222222222222222222222222222222222333333333333333333333333333333333333333333333333333333333333333333333334......
        # {\n"key_4_UF0aJJ6v": "value_1", \n"key_2_Hd0t": [\n"value_16", \n"value_8", \n"value_8", \n"value_15", \n"value_14", \n"value_19",...... 
        # 1 222222222222222222222222222222 2222222222222222 3333333333333 333333333333 333333333333 3333333333333 3333333333333 3333333333333...... 
        '''
    curr_ch = j_str[curr_seq]
    next_ch = j_str[curr_seq + 1]
    cond = 0
    for i in range(1,block_op_pairs_dict.__len__()+1):
        if(curr_ch == block_op_pairs_dict[i][0]):
            if(next_ch == block_op_pairs_dict[i][1]):
                next_lv = curr_lv               
            else:
                next_lv = curr_lv + 1
            cond = 1
            break
        elif(curr_ch == block_op_pairs_dict[i][1]):
            if(block.is_rop(next_ch,block_op_pairs_dict)):
                next_lv = curr_lv - 1
            else:
                next_lv = curr_lv
            cond = 1
            break
        else:
            pass
    if(cond == 1):
        pass
    elif(block.is_rop(next_ch,block_op_pairs_dict)):
        next_lv = curr_lv - 1
    else:    
        next_lv = curr_lv
    curr_lv = next_lv
    curr_seq = curr_seq + 1
    return(curr_lv,curr_lv,curr_seq)



import efuntool.ebooltool as ebtl
import efuntool.efuntool as eftl
import elist.elist as elel
import itertools
import estring.estring as eses
from xdict.jprint import parr


def creat_apifix(s):
    arr = ebtl.combinate_all(s)
    arr = elel.mapv(arr,elel.join,[""])
    arr.reverse()
    arr.remove('')
    return(arr)


def creat_api_names(pre,verb,suf):
    params = creat_apifix(pre)
    rtrns = creat_apifix(suf)
    tl = list(itertools.product(params,rtrns))
    tl = elel.mapv(rtrns,lambda ele:("",ele)) if(len(params)==0) else tl
    tl = elel.mapv(params,lambda ele:(ele,"")) if(len(rtrns)==0) else tl
    names = elel.mapv(tl,lambda ele:ele[0]+verb+ele[1])
    return(names)

def creat_empty_def(name):
    l0 = "def " + name +"():"
    l1 = "    rslt="
    l2 = "    return(rslt)"
    return(l0+"\n"+l1+"\n"+l2+"\n")

def creat_apis_via_names(api_names):
    arr = []
    for name in api_names:
        cd = creat_empty_def(name)
        print(cd)
        print("\n")
        arr.append(cd)
    return(arr)

def creat_apis_from_verb(pre,verb,suf):
    names = creat_api_names(pre,verb,suf)
    apis = creat_apis_via_names(names)
    return(apis)

def str2arr(s):
    s = s.strip("\n").strip(" ").strip("\n").strip(" ")
    s = s.replace("\n\n","\n")
    s = s.replace("\n\n","\n")
    arr = s.split("\n")
    return(arr)


def creat_apis_from_strblk(s):
    api_names = str2arr(s)
    return(creat_apis_via_names(api_names))



###

def qw(s,comma=',',quote='"'):
    l = s.split(comma)
    rslt = ''
    for i in range(0,l.__len__()):
        ele = l[i]
        nele = quote+ele+quote
        rslt = rslt + nele + comma
    rslt = eses.rstrip(rslt,comma,1)
    return(rslt)


def indent(code,*args,**kwargs):
    '''
    '''
    indent = eftl.optional_arg("    ",*args)
    indent = ("    "*indent) if(isinstance(indent,int)) else indent
    line_sp = '\n'
    lines = code.split(line_sp)
    ncode = ''
    length = lines.__len__()
    for i in range(0,length-1):
        line = indent + lines[i] + line_sp
        ncode = ncode + line
    ncode = ncode + indent + lines[length-1]
    return(ncode)


####

def creat_onewrap(name,wrapper):
    l0 = "@engine."+wrapper
    l1 = "def " + name + "(d):"
    l2 = "    return(" + "engine."+ name +"(d)" + ")"
    return(l0+"\n"+l1+"\n"+l2+"\n")

def creat_wraps(s,wrapper):
    arr = str2arr(s)
    cds = elel.mapv(arr,creat_onewrap,[wrapper])
    parr(cds)
    return(cds)


##cProfile###

def profile(code):
    '''
        dummy = profile('print("sss")')
    '''
    import cProfile
    import pstats
    from io import StringIO
    cProfile.run(code,"info")
    s = StringIO()
    sortby = 'cumulative'
    ps = pstats.Stats("info", stream=s).sort_stats(sortby)
    ps.print_stats()
    info = s.getvalue()
    print(info)
    return(info)

########

#@fn@            function-name

def fntem_rplc(fn,tem):
    '''
        replace  @fn@
    '''
    s = tem.replace("@fn@",fn)
    return(s)


def rtrntem_rplc(rtrn,tem):
    '''
        replace  @rtrn@
    '''
    s = tem.replace("@rtrn@",rtrn)
    return(s)

#####






import xdict.CrtableLib.crtable as xcr
colnameslist = ['name','chapter','sentence','startloc','endloc']
keynameslist = ['name']
table = {}
table[0] = ['姜维', 98, '姜维', 3215, 3217]
table[1] = ['姜维', 100, '姜维', 2476, 2478]
table[2] = ['姜维', 101, '姜维', 1699, 1701]
crtb = xcr.crtable(colnameslist = colnameslist,table=table,keynameslist = keynameslist)
print(crtb.__str__())


#+++++++++++++++++++++++++++++++++++++++
#|name|chapter|sentence|startloc|endloc|
#+++++++++++++++++++++++++++++++++++++++
#|姜维|     98|    姜维|    3215|  3217|
#+++++++++++++++++++++++++++++++++++++++
#|姜维|    100|    姜维|    2476|  2478|
#+++++++++++++++++++++++++++++++++++++++
#|姜维|    101|    姜维|    1699|  1701|
#+++++++++++++++++++++++++++++++++++++++


import xdict.CrtableLib.crtable as xcr

ROWs = {
         0: {0: 'TBqESO', 1: 'jVymZLRRhj', 2: 'VlWVOMszE'},
         1: {0: 'apxjPB', 1: 'iSdHTH', 2: 'WuqefeO'},
         2: {0: 'alYhP', 1: 'lOST', 2: 'anQOLA'}
        }

display_COLs = xcr.display_table_via_rows(ROWs,returned=True)


colormatrix = {
      0: {0: 'yellow', 1: 'green', 2: 'blue'},
      1: {0: 'red', 1: 'yellow', 2: 'green'},
      2: {0: 'blue', 1: 'yellow', 2: 'green'}
     }

COLs = xcr.rows_to_cols(ROWs)


xcr.display_table_via_rows(ROWs,colormatrix=colormatrix,returned=True)


m = [
    [1,2,3,4,5],
    [2,2,2,2,2],
    [55,6,7,7,8]
]

import xdict.CrtableLib.crtable as xcr
xcr.shmat(m)

#s =''
#color_sec ={}
#cursor = 0
#for j in range(0,COLs.__len__()):
#    s = s + display_COLs[j][i]
#    length = display_COLs[j][i].__len__()
#    color_sec[j+1] = (cursor,cursor+length-1,colormatrix[i][j])
#    cursor = cursor+length

#boundary = '+' * len(s)
#s = s +'\n'+boundary
#color_sec[COLs.__len__()+1] = (cursor,cursor+boundary.__len__(),"white")
#console.paint(s,color_sec=color_sec)



from xdict.ide import *
from efuntool.efuntool import dflt_sysargv


def main():
    pre = dflt_sysargv("",1)
    verb = dflt_sysargv("return",2)
    suf = dflt_sysargv("",3)
    creat_apis_from_verb(pre,verb,suf)




import estring.estring as eses

def html_number_escape_char(ch):
    '''
        >>> es = html_number_escape_char('a')
        >>> es
        '&#97;'
        >>> html.unescape(es)
        'a'
        >>> es = html_number_escape_char('用')
        >>> es
        '&#29992;'
        >>> html.unescape(es)
        '用'
        >>>
    '''
    num = int(eses.str2us(ch,style="js")[2:],16)
    escaped = '&#' + str(num) +';'
    return(escaped)


def html_number_escape_str(s):
    '''
        >>>
        >>> ess = html_number_escape_str('加强武器')
        >>> ess
        '&#21152;&#24378;&#27494;&#22120;'
        >>> html.unescape(ess)
        '加强武器'
        >>> ess = html_number_escape_str('xyzw')
        >>> ess
        '&#120;&#121;&#122;&#119;'
        >>> html.unescape(ess)
        'xyzw'
        >>>
    '''
    escaped = ''
    for i in range(0,s.__len__()):
        esch = html_number_escape_char(s[i])
        escaped = escaped+esch
    return(escaped)







def get_quotes_pairs(pairs_str):
    '''
        # operators ,such as {} [] ()  ......could be user-defined
        # >>> get_block_op_pairs("{}[]")
        # {1: ('{', '}'), 2: ('[', ']')}
        # >>> get_block_op_pairs("{}[]()")
        # {1: ('{', '}'), 2: ('[', ']'), 3: ('(', ')')}
        # >>> get_block_op_pairs("{}[]()<>")
        # {1: ('{', '}'), 2: ('[', ']'), 3: ('(', ')'), 4: ('<', '>')}
    '''
    pairs_str_len = pairs_str.__len__()
    pairs_len = pairs_str_len // 2
    pairs_dict = {}
    for i in range(1,pairs_len +1):
        pairs_dict[i] = pairs_str[i*2-2],pairs_str[i*2-1]
    return(pairs_dict)




def get_lrquotes(quotes_pairs_dict):
    lquotes = []
    rquotes = []
    quotes = []
    for i in range(1,quotes_pairs_dict.__len__()+1):
        lquotes.append(quotes_pairs_dict[i][0])
        rquotes.append(quotes_pairs_dict[i][1])
        quotes.append(quotes_pairs_dict[i][0])
        quotes.append(quotes_pairs_dict[i][1])
    return((lquotes,rquotes,quotes))







from xdict import utils
from xdict import jprint
from xdict import cmdline
from xdict import hdict_object
from xdict import hdict_xml


class description():
    def __init__(self,**kwargs):
        if('readable_path' in kwargs):
            self.readable_path = kwargs['readable_path']
        else:
            self.readable_path = None
        if('pathlist' in kwargs):
            self.pathlist = kwargs['pathlist']
        else:
            self.pathlist = None
        if('depth' in kwargs):
            self.depth = kwargs['depth']
        else:
            self.depth = None
        if('breadth' in kwargs):
            self.breadth = kwargs['breadth']
        else:
            self.breadth = None
        if('siblings_seq' in kwargs):
            self.siblings_seq = kwargs['siblings_seq']
        else:
            self.siblings_seq = None
        if('tag' in kwargs):
            self.tag = kwargs['tag']
        else:
            self.tag = None
        if('attrib' in kwargs):
            self.attrib = kwargs['attrib']
        else:
            self.attrib = None
        if('text' in kwargs):
            self.text = kwargs['text']
        else:
            self.text = None
        if('type' in kwargs):
            self.type = kwargs['type']
        else:
            self.type = None
        if('leaf' in kwargs):
            self.leaf = kwargs['leaf']
        else:
            self.leaf = None
        if('leaf_sons' in kwargs):
            self.leaf_sons = kwargs['leaf_sons']
        else:
            self.leaf_sons = None
        if('nonleaf_sons' in kwargs):
            self.nonleaf_sons = kwargs['nonleaf_sons']
        else:
            self.nonleaf_sons = None
        if('leaf_descendants' in kwargs):
            self.leaf_descendants = kwargs['leaf_descendants']
        else:
            self.leaf_descendants = None
        if('nonleaf_descendants' in kwargs):
            self.nonleaf_descendants = kwargs['nonleaf_descendants']
        else:
            self.nonleaf_descendants = None
    def __dir__(self):
        l = ['readable_path','pathlist','depth','breadth','siblings_seq','tag','attrib','text','type','leaf','leaf_sons','nonleaf_sons','leaf_descendants','nonleaf_descendants']
        jprint.pobj(l)
        return(l)
    def __repr__(self):
        printed_str = ''
        l = ['readable_path','pathlist','depth','breadth','siblings_seq','tag','attrib','text','type','leaf','leaf_sons','nonleaf_sons','leaf_descendants','nonleaf_descendants']
        for i in range(0,l.__len__()):
            s = '{0} : {1}'.format(l[i],str(self.__getattribute__(l[i])))
            printed_str = ''.join((printed_str,s,'\n'))
        return(printed_str)

class nodedescription():
    def __init__(self,**kwargs):
        if('readable_path' in kwargs):
            self.readable_path = kwargs['readable_path']
        else:
            self.readable_path = None
        if('pathlist' in kwargs):
            self.pathlist = kwargs['pathlist']
        else:
            self.pathlist = None
        if('depth' in kwargs):
            self.depth = kwargs['depth']
        else:
            self.depth = None
        if('breadth' in kwargs):
            self.breadth = kwargs['breadth']
        else:
            self.breadth = None
        if('siblings_seq' in kwargs):
            self.siblings_seq = kwargs['siblings_seq']
        else:
            self.siblings_seq = None
        if('tag' in kwargs):
            self.tag = kwargs['tag']
        else:
            self.tag = None
        if('attrib' in kwargs):
            self.attrib = kwargs['attrib']
        else:
            self.attrib = None
        if('text' in kwargs):
            self.text = kwargs['text']
        else:
            self.text = None
        if('type' in kwargs):
            self.type = kwargs['type']
        else:
            self.type = None
        if('leaf' in kwargs):
            self.leaf = kwargs['leaf']
        else:
            self.leaf = None
        if('leaf_sons' in kwargs):
            self.leaf_sons = kwargs['leaf_sons']
        else:
            self.leaf_sons = None
        if('nonleaf_sons' in kwargs):
            self.nonleaf_sons = kwargs['nonleaf_sons']
        else:
            self.nonleaf_sons = None
        if('leaf_descendants' in kwargs):
            self.leaf_descendants = kwargs['leaf_descendants']
        else:
            self.leaf_descendants = None
        if('nonleaf_descendants' in kwargs):
            self.nonleaf_descendants = kwargs['nonleaf_descendants']
        else:
            self.nonleaf_descendants = None
        if('lsib' in kwargs):
            self.lsib = kwargs['lsib']
        else:
            self.lsib = None
        if('rsib' in kwargs):
            self.rsib = kwargs['rsib']
        else:
            self.rsib = None
        if('lcin' in kwargs):
            self.lcin = kwargs['lcin']
        else:
            self.lcin = None
        if('rcin' in kwargs):
            self.rcin = kwargs['rcin']
        else:
            self.rcin = None
    def __dir__(self):
        l = ['readable_path','pathlist','depth','breadth','siblings_seq','tag','attrib','text','type','leaf','leaf_sons','nonleaf_sons','leaf_descendants','nonleaf_descendants','lsib','rsib','lcin','rcin']
        jprint.pobj(l)
        return(l)
    def __repr__(self):
        printed_str = ''
        l = ['readable_path','pathlist','depth','breadth','siblings_seq','tag','attrib','text','type','leaf','leaf_sons','nonleaf_sons','leaf_descendants','nonleaf_descendants','lsib','rsib','lcin','rcin']
        for i in range(0,l.__len__()):
            s = '{0} : {1}'.format(l[i],str(self.__getattribute__(l[i])))
            printed_str = ''.join((printed_str,s,'\n'))
        return(printed_str)

class hdict():
    '''
        from xdict.HdictLib import hdict
        from xdict.jprint import pobj
        currd = {'UseSpeedLimits': None, 'RuleIDs': [11516125, 11516163, 11516164], 'Displays': [{'RequiresHRBelt': None, 'Type': 5, 'Row1': {'Row': 37, 'RuleID': None}, 'Row2': {'Row': None, 'RuleID': 11516125}, 'Views': [{'Row': None, 'RuleID': 11516163}]}, {'RequiresHRBelt': None, 'Type': 5, 'Row1': {'Row': 39, 'RuleID': None}, 'Row2': {'Row': 41, 'RuleID': None}, 'Views': [{'Row': 40, 'RuleID': None}]}, {'RequiresHRBelt': None, 'Type': 5, 'Row1': {'Row': 38, 'RuleID': None}, 'Row2': {'Row': 68, 'RuleID': None}, 'Views': [{'Row': 10, 'RuleID': None}]}, {'RequiresHRBelt': None, 'Type': 5, 'Row1': {'Row': 48, 'RuleID': None}, 'Row2': {'Row': 49, 'RuleID': None}, 'Views': [{'Row': 50, 'RuleID': None}]}, {'RequiresHRBelt': None, 'Type': 5, 'Row1': {'Row': 51, 'RuleID': None}, 'Row2': {'Row': 52, 'RuleID': None}, 'Views': [{'Row': 53, 'RuleID': None}]}, {'RequiresHRBelt': None, 'Type': 5, 'Row1': {'Row': 54, 'RuleID': None}, 'Row2': {'Row': 56, 'RuleID': None}, 'Views': [{'Row': 57, 'RuleID': None}]}, {'RequiresHRBelt': None, 'Type': 5, 'Row1': {'Row': 58, 'RuleID': None}, 'Row2': {'Row': 59, 'RuleID': None}, 'Views': [{'Row': 12, 'RuleID': None}]}, {'RequiresHRBelt': None, 'Type': 5, 'Row1': {'Row': None, 'RuleID': 11516164}, 'Row2': {'Row': 4, 'RuleID': None}, 'Views': [{'Row': 20, 'RuleID': None}]}], 'AutomaticLogRecording': None, 'AutoPause': None, 'LoggedRuleIDs': [11516163, 11516164, 11516125]}
        hdict1 = hdict.hdict(object=currd)
        dir(hdict1)
        
        hdict1.depth
        hdict.widths
        print(hdict1.html)
        hdict1.showall()
        hdict1.search('uleIDs')
        
        
    '''
    def __init__(self,**kwargs):
        if('object' in kwargs):
            self.object = kwargs['object']
            hspr = hdict_object.obj_to_hdict(self.object)
            self.cmdt = cmdline.cmdict(dict=self.object)
            self.html = hdict_object.hspr_to_xml(hspr)
            self.orig = 'object'
        elif('html' in kwargs):
            self.html = kwargs['html']
            hspr = hdict_xml.html_to_hdict(html_text=self.html)
            self.cmdt = cmdline.cmdict(html_text=self.html)
            self.object = hdict_object.hdict_to_obj(hspr['hdict'],hspr['sdict'],hspr['prdict'])
            self.orig = 'html'
        elif('cmdline' in kwargs):
            self.cmdt = kwargs['cmdline']
            cmdlines_full_dict = {}
            cmdlines_full_dict['cmds'] = self.cmdt.cmdlines
            cmdlines_full_dict['attribs'] = self.cmdt.attribs
            cmdlines_full_dict['results'] = self.cmdt.results
            #在cmdline.py 中加入代码支持cmds 是 pathlists 的情况
            hspr = cmdline.cmdlines_full_dict_to_hdict(cmdlines_full_dict)
            self.html = hdict_object.hspr_to_xml(hspr)
            self.object = hdict_object.hdict_to_obj(hspr['hdict'],hspr['sdict'],hspr['prdict'])
            self.orig = 'cmdline'
        else:
            raise Exception("Invalid INIT!", kwargs)
        self.hdict = hspr['hdict']
        self.sdict = hspr['sdict']
        self.prdict = hspr['prdict']
        self.depth = self.sdict.__len__()
        self.widths = []
        for i in range(0,self.depth):
            self.widths.append(self.sdict[i].__len__())
    def __dir__(self):
        l = ['depth','widths','node','showall','search','html','object']
        jprint.pobj(l)
        return(l)
    def __repr__(self):
        if(self.orig == 'object'):
            return(self.object.__repr__())
        elif(self.orig == 'html'):
            return(html)
        elif(self.orig == 'cmdline'):
            jprint.pobj(self.cmdt.pathlists,fixed_indent = 1)
            print('\n')
            jprint.pobj(self.cmdt.attribs,fixed_indent = 1)
            print('\n')
            jprint.pobj(self.cmdt.results,fixed_indent = 1)
            print('\n')
            return(self.cmdt.pathlists.__repr__() + self.cmdt.attribs.__repr__() + self.cmdt.results.__repr__())
        else:
            raise Exception("Invalid INIT!",'not object,not html_text,not cmdline')
    def showall(self,**kwargs):
        if('mode' in kwargs):
            mode = kwargs['mode']
        else:
            mode = 'readable_path'
        if(mode == 'pathlist'):
            jprint.pobj(self.cmdt.pathlists,fixed_indent = 1)
        elif(mode == 'cmdline'):
            jprint.pobj(self.cmdt.cmdlines,fixed_indent = 1)
        else:
            readables = {}
            for i in range(0,self.cmdt.pathlists.__len__()):
                readable = hdict_object.path_list_to_console_key_str(self.cmdt.pathlists[i])
                readables[i] = readable
            jprint.pobj(readables,fixed_indent = 1)
    def search(self,cmd_str):
        return(self.cmdt.__getitem__(cmd_str))
    def getlsib(self,pathlist):
        hdict_pathlist = hdict_object.orig_obj_path_to_hdict_path(self.prdict,pathlist)
        breadth_path = hdict_object.hdict_path_to_breadth_path(self.hdict,hdict_pathlist)
        depth,breadth = hdict_object.breadth_path_to_sdict_location(breadth_path)
        lsib_pathlist = self.sdict[depth][breadth]['orig_lsib_path']
        if(lsib_pathlist == []):
            lsib_pathlist = None
            lsib_readable_path = None
            lsib_depth = None
            lsib_breadth = None
            lsib_siblings_seq = None
            lsib_tag = None
            lsib_attrib = None
            lsib_text = None
            lsib_type = None
            lsib_leaf = None
            #####
            lsib_leaf_sons = None 
            lsib_nonleaf_sons = None
            lsib_leaf_descendants = None
            lsib_nonleaf_descendants = None
            #####
        else:
            lsib_readable_path = hdict_object.path_list_to_console_key_str(lsib_pathlist)
            lsib_hdict_pathlist = hdict_object.orig_obj_path_to_hdict_path(self.prdict,lsib_pathlist)
            lsib_breadth_path = hdict_object.hdict_path_to_breadth_path(self.hdict,lsib_hdict_pathlist)
            lsib_depth,lsib_breadth = hdict_object.breadth_path_to_sdict_location(lsib_breadth_path)
            lsib_siblings_seq = self.sdict[lsib_depth][lsib_breadth]['siblings_seq_path'][-1]
            lsib_tag = hdict_object.hdict_get_value(self.hdict,lsib_pathlist)['tag']
            lsib_attrib = self.sdict[lsib_depth][lsib_breadth]['attrib']
            lsib_text = self.sdict[lsib_depth][lsib_breadth]['text']
            lsib_type = self.sdict[lsib_depth][lsib_breadth]['type']
            lsib_leaf = self.sdict[lsib_depth][lsib_breadth]['leaf']
            ##########################################
            lsib_leaf_sons = self.sdict[lsib_depth][lsib_breadth]['leaf_son_pathlists']
            lsib_nonleaf_sons = self.sdict[lsib_depth][lsib_breadth]['non_leaf_son_pathlists']
            lsib_leaf_descendants = self.sdict[lsib_depth][lsib_breadth]['leaf_descendant_pathlists']
            lsib_nonleaf_descendants = self.sdict[lsib_depth][lsib_breadth]['non_leaf_descendant_pathlists']
            ###########################################
        desc = description(readable_path=lsib_readable_path,pathlist=lsib_pathlist,depth=lsib_depth,breadth=lsib_breadth,siblings_seq=lsib_siblings_seq,tag=lsib_tag,attrib=lsib_attrib,text=lsib_text,type=lsib_type,leaf=lsib_leaf,leaf_sons=lsib_leaf_sons,nonleaf_sons=lsib_nonleaf_sons,leaf_descendants=lsib_leaf_descendants,nonleaf_descendants=lsib_nonleaf_descendants)
        return(desc)
    def getrsib(self,pathlist):
        hdict_pathlist = hdict_object.orig_obj_path_to_hdict_path(self.prdict,pathlist)
        breadth_path = hdict_object.hdict_path_to_breadth_path(self.hdict,hdict_pathlist)
        depth,breadth = hdict_object.breadth_path_to_sdict_location(breadth_path)
        rsib_pathlist = self.sdict[depth][breadth]['orig_rsib_path']
        if(rsib_pathlist == []):
            rsib_pathlist = None
            rsib_readable_path = None
            rsib_depth = None
            rsib_breadth = None
            rsib_siblings_seq = None
            rsib_tag = None
            rsib_attrib = None
            rsib_text = None
            rsib_type = None
            rsib_leaf = None
            #####
            rsib_leaf_sons = None 
            rsib_nonleaf_sons = None
            rsib_leaf_descendants = None
            rsib_nonleaf_descendants = None
            #####
        else:
            rsib_readable_path = hdict_object.path_list_to_console_key_str(rsib_pathlist)
            rsib_hdict_pathlist = hdict_object.orig_obj_path_to_hdict_path(self.prdict,rsib_pathlist)
            rsib_breadth_path = hdict_object.hdict_path_to_breadth_path(self.hdict,rsib_hdict_pathlist)
            rsib_depth,rsib_breadth = hdict_object.breadth_path_to_sdict_location(rsib_breadth_path)
            rsib_siblings_seq = self.sdict[rsib_depth][rsib_breadth]['siblings_seq_path'][-1]
            rsib_tag = hdict_object.hdict_get_value(self.hdict,rsib_pathlist)['tag']
            rsib_attrib = self.sdict[rsib_depth][rsib_breadth]['attrib']
            rsib_text = self.sdict[rsib_depth][rsib_breadth]['text']
            rsib_type = self.sdict[rsib_depth][rsib_breadth]['type']
            rsib_leaf = self.sdict[rsib_depth][rsib_breadth]['leaf']
            ##########################################
            rsib_leaf_sons = self.sdict[rsib_depth][rsib_breadth]['leaf_son_pathlists']
            rsib_nonleaf_sons = self.sdict[rsib_depth][rsib_breadth]['non_leaf_son_pathlists']
            rsib_leaf_descendants = self.sdict[rsib_depth][rsib_breadth]['leaf_descendant_pathlists']
            rsib_nonleaf_descendants = self.sdict[rsib_depth][rsib_breadth]['non_leaf_descendant_pathlists']
            ###########################################
        desc = description(readable_path=rsib_readable_path,pathlist=rsib_pathlist,depth=rsib_depth,breadth=rsib_breadth,siblings_seq=rsib_siblings_seq,tag=rsib_tag,attrib=rsib_attrib,text=rsib_text,type=rsib_type,leaf=rsib_leaf,leaf_sons=rsib_leaf_sons,nonleaf_sons=rsib_nonleaf_sons,leaf_descendants=rsib_leaf_descendants,nonleaf_descendants=rsib_nonleaf_descendants)
        return(desc)
    def getlcin(self,pathlist):
        hdict_pathlist = hdict_object.orig_obj_path_to_hdict_path(self.prdict,pathlist)
        breadth_path = hdict_object.hdict_path_to_breadth_path(self.hdict,hdict_pathlist)
        depth,breadth = hdict_object.breadth_path_to_sdict_location(breadth_path)
        lcin_pathlist = self.sdict[depth][breadth]['orig_lcin_path']
        if(lcin_pathlist == []):
            lcin_pathlist = None
            lcin_readable_path = None
            lcin_depth = None
            lcin_breadth = None
            lcin_siblings_seq = None
            lcin_tag = None
            lcin_attrib = None
            lcin_text = None
            lcin_type = None
            lcin_leaf = None
            #####
            lcin_leaf_sons = None 
            lcin_nonleaf_sons = None
            lcin_leaf_descendants = None
            lcin_nonleaf_descendants = None
            #####
        else:
            lcin_readable_path = hdict_object.path_list_to_console_key_str(lcin_pathlist)
            lcin_hdict_pathlist = hdict_object.orig_obj_path_to_hdict_path(self.prdict,lcin_pathlist)
            lcin_breadth_path = hdict_object.hdict_path_to_breadth_path(self.hdict,lcin_hdict_pathlist)
            lcin_depth,lcin_breadth = hdict_object.breadth_path_to_sdict_location(lcin_breadth_path)
            lcin_siblings_seq = self.sdict[lcin_depth][lcin_breadth]['siblings_seq_path'][-1]
            lcin_tag = hdict_object.hdict_get_value(self.hdict,lcin_pathlist)['tag']
            lcin_attrib = self.sdict[lcin_depth][lcin_breadth]['attrib']
            lcin_text = self.sdict[lcin_depth][lcin_breadth]['text']
            lcin_type = self.sdict[lcin_depth][lcin_breadth]['type']
            lcin_leaf = self.sdict[lcin_depth][lcin_breadth]['leaf']
            ##########################################
            lcin_leaf_sons = self.sdict[lcin_depth][lcin_breadth]['leaf_son_pathlists']
            lcin_nonleaf_sons = self.sdict[lcin_depth][lcin_breadth]['non_leaf_son_pathlists']
            lcin_leaf_descendants = self.sdict[lcin_depth][lcin_breadth]['leaf_descendant_pathlists']
            lcin_nonleaf_descendants = self.sdict[lcin_depth][lcin_breadth]['non_leaf_descendant_pathlists']
            ###########################################
        desc = description(readable_path=lcin_readable_path,pathlist=lcin_pathlist,depth=lcin_depth,breadth=lcin_breadth,siblings_seq=lcin_siblings_seq,tag=lcin_tag,attrib=lcin_attrib,text=lcin_text,type=lcin_type,leaf=lcin_leaf,leaf_sons=lcin_leaf_sons,nonleaf_sons=lcin_nonleaf_sons,leaf_descendants=lcin_leaf_descendants,nonleaf_descendants=lcin_nonleaf_descendants)
        return(desc)
    def getrcin(self,pathlist):
        hdict_pathlist = hdict_object.orig_obj_path_to_hdict_path(self.prdict,pathlist)
        breadth_path = hdict_object.hdict_path_to_breadth_path(self.hdict,hdict_pathlist)
        depth,breadth = hdict_object.breadth_path_to_sdict_location(breadth_path)
        rcin_pathlist = self.sdict[depth][breadth]['orig_rcin_path']
        if(rcin_pathlist == []):
            rcin_pathlist = None
            rcin_readable_path = None
            rcin_depth = None
            rcin_breadth = None
            rcin_siblings_seq = None
            rcin_tag = None
            rcin_attrib = None
            rcin_text = None
            rcin_type = None
            rcin_leaf = None
            #####
            rcin_leaf_sons = None 
            rcin_nonleaf_sons = None
            rcin_leaf_descendants = None
            rcin_nonleaf_descendants = None
            #####
        else:
            rcin_readable_path = hdict_object.path_list_to_console_key_str(rcin_pathlist)
            rcin_hdict_pathlist = hdict_object.orig_obj_path_to_hdict_path(self.prdict,rcin_pathlist)
            rcin_breadth_path = hdict_object.hdict_path_to_breadth_path(self.hdict,rcin_hdict_pathlist)
            rcin_depth,rcin_breadth = hdict_object.breadth_path_to_sdict_location(rcin_breadth_path)
            rcin_siblings_seq = self.sdict[rcin_depth][rcin_breadth]['siblings_seq_path'][-1]
            rcin_tag = hdict_object.hdict_get_value(self.hdict,rcin_pathlist)['tag']
            rcin_attrib = self.sdict[rcin_depth][rcin_breadth]['attrib']
            rcin_text = self.sdict[rcin_depth][rcin_breadth]['text']
            rcin_type = self.sdict[rcin_depth][rcin_breadth]['type']
            rcin_leaf = self.sdict[rcin_depth][rcin_breadth]['leaf']
            ##########################################
            rcin_leaf_sons = self.sdict[rcin_depth][rcin_breadth]['leaf_son_pathlists']
            rcin_nonleaf_sons = self.sdict[rcin_depth][rcin_breadth]['non_leaf_son_pathlists']
            rcin_leaf_descendants = self.sdict[rcin_depth][rcin_breadth]['leaf_descendant_pathlists']
            rcin_nonleaf_descendants = self.sdict[rcin_depth][rcin_breadth]['non_leaf_descendant_pathlists']
            ###########################################
        desc = description(readable_path=rcin_readable_path,pathlist=rcin_pathlist,depth=rcin_depth,breadth=rcin_breadth,siblings_seq=rcin_siblings_seq,tag=rcin_tag,attrib=rcin_attrib,text=rcin_text,type=rcin_type,leaf=rcin_leaf,leaf_sons=rcin_leaf_sons,nonleaf_sons=rcin_nonleaf_sons,leaf_descendants=rcin_leaf_descendants,nonleaf_descendants=rcin_nonleaf_descendants)
        return(desc)
    def node(self,pathlist):
        hdict_pathlist = hdict_object.orig_obj_path_to_hdict_path(self.prdict,pathlist)
        breadth_path = hdict_object.hdict_path_to_breadth_path(self.hdict,hdict_pathlist)
        depth,breadth = hdict_object.breadth_path_to_sdict_location(breadth_path)
        node_pathlist = self.sdict[depth][breadth]['orig_obj_path']
        if(node_pathlist == []):
            node_pathlist = None
            node_readable_path = None
            node_depth = None
            node_breadth = None
            node_siblings_seq = None
            node_tag = None
            node_attrib = None
            node_text = None
            node_type = None
            node_leaf = None
            #####
            node_leaf_sons = None 
            node_nonleaf_sons = None
            node_leaf_descendants = None
            node_nonleaf_descendants = None
            #####
            node_lsib = None
            node_rsib = None
            node_lcin = None
            node_rcin = None
            #####
        else:
            node_readable_path = hdict_object.path_list_to_console_key_str(node_pathlist)
            node_hdict_pathlist = hdict_object.orig_obj_path_to_hdict_path(self.prdict,node_pathlist)
            node_breadth_path = hdict_object.hdict_path_to_breadth_path(self.hdict,node_hdict_pathlist)
            node_depth,node_breadth = hdict_object.breadth_path_to_sdict_location(node_breadth_path)
            node_siblings_seq = self.sdict[node_depth][node_breadth]['siblings_seq_path'][-1]
            node_tag = hdict_object.hdict_get_value(self.hdict,node_pathlist)['tag']
            node_attrib = self.sdict[node_depth][node_breadth]['attrib']
            node_text = self.sdict[node_depth][node_breadth]['text']
            node_type = self.sdict[node_depth][node_breadth]['type']
            node_leaf = self.sdict[node_depth][node_breadth]['leaf']
            ##########################################
            node_leaf_sons = self.sdict[node_depth][node_breadth]['leaf_son_pathlists']
            node_nonleaf_sons = self.sdict[node_depth][node_breadth]['non_leaf_son_pathlists']
            node_leaf_descendants = self.sdict[node_depth][node_breadth]['leaf_descendant_pathlists']
            node_nonleaf_descendants = self.sdict[node_depth][node_breadth]['non_leaf_descendant_pathlists']
            ###########################################
            #####
            node_lsib = self.getlsib(pathlist)
            node_rsib = self.getrsib(pathlist)
            node_lcin = self.getlcin(pathlist)
            node_rcin = self.getrcin(pathlist)
            #####
        desc = nodedescription(readable_path=node_readable_path,pathlist=node_pathlist,depth=node_depth,breadth=node_breadth,siblings_seq=node_siblings_seq,tag=node_tag,attrib=node_attrib,text=node_text,type=node_type,leaf=node_leaf,leaf_sons=node_leaf_sons,nonleaf_sons=node_nonleaf_sons,leaf_descendants=node_leaf_descendants,nonleaf_descendants=node_nonleaf_descendants,lsib=node_lsib,rsib=node_rsib,lcin=node_lcin,rcin=node_rcin)
        return(desc)




# obj - obj_to_hdict ->hdict,sdict,prdict 
# hdict - hdict_get_paths_relationship -> prdict
# hdict sdict(internal state),prdict - hdict_fullfill_sdict -> sdict
# hdict - get_sdict_and_prdict_from_hdict -> sdict,prdict
# any_kind_of_path  - prdict -> breadth_path - breadth_path_to_sdict_location -> sdict_location
# hdict,sdict,prdict - hdict_to_obj -> obj 



import copy
import json
import cgi
from xdict import utils
import ltdict.ltdict as ltdict

def obj_to_hdict(obj,**kwargs):
    def add_dict_tree_entry(hdict,path,value,n2s=0,s2n=0):
        utils.dict_setdefault_via_path_list(hdict,path,n2s=n2s,s2n=s2n)
        utils.dict_setitem_via_path_list(hdict,path,value,n2s=n2s,s2n=s2n)
    if('debug' in kwargs):
        debug = kwargs['debug']
    else:
        debug = 0
    if('deepcopy' in kwargs):
        deepcopy = kwargs['deepcopy']
    else:
        deepcopy = 1
    if('save' in kwargs):
        save = kwargs['save']
    else:
        save = 0
    if('n2s' in kwargs):
        n2s = kwargs['n2s']
    else:
        n2s = 0
    if('s2n' in kwargs):
        s2n = kwargs['n2s']
    else:
        s2n = 0
    if('json' in kwargs):
        js = kwargs['json']
    else:
        js = 0
    if('get_sdict' in kwargs):
        get_sdict = kwargs['get_sdict']
    else:
        get_sdict =1
    if('sdict' in kwargs):
        sdict = kwargs['sdict']
    else:
        sdict = {}
    hdict = {}
    if(deepcopy):
        new_obj = copy.deepcopy(obj)
    else:
        new_obj = obj
    if(debug):
        print("root:{0}".format(new_obj))
    else:
        pass
    unhandled = []
    if(utils.is_list(new_obj)|utils.is_tuple(new_obj)):
        for i in range(0,new_obj.__len__()):
            unhandled.append({'node':new_obj[i],'path':[i],'tag':i,'siblings_seq':i,'breadth':i,'depth':0,'orig_obj_path':[i],'breadth_path':[i]})
    elif(utils.is_set(new_obj)):
        i = 0
        for ele in new_obj:
            unhandled.append({'node':ele,'path':[i],'tag':i,'siblings_seq':i,'breadth':i,'depth':0,'orig_obj_path':[i],'breadth_path':[i],'orig_obj_path_loose':[]})
            i = i + 1
    elif(utils.is_dict(new_obj)):
        seq = 0
        for key in new_obj:
            unhandled.append({'node':new_obj[key],'path':[seq],'tag':key,'siblings_seq':seq,'breadth':seq,'depth':0,'orig_obj_path':[key],'breadth_path':[seq]})
            seq = seq + 1
    else:
        hdict[0] = {}
        hdict[0]['tag'] = ''
        hdict[0]['attrib'] = {}
        hdict[0]['attrib']['type'] = utils.get_obj_type_name(new_obj)
        hdict[0]['text'] = str(new_obj)
        hdict[0]['depth'] = 0
        hdict[0]['breadth'] = 0
        hdict[0]['breadth_path'] = 0
        hdict[0]['siblings_seq'] = 0
        hdict[0]['path'] = [0]
        hdict[0]['children'] = {}
        return(hdict)
    depth = 0
    while(unhandled.__len__()>0):
        if(debug):
            print("--------------depth:{0}---------------------".format(depth))
        else:
            pass
        next_unhandled = []
        sdict[depth] = {}
        for i in range(0,unhandled.__len__()):
            parent = unhandled[i]
            parent_path = unhandled[i]['path']
            parent_path_path = copy.copy(parent_path)
            parent_path_path.append('path')
            add_dict_tree_entry(hdict,parent_path_path,parent_path,s2n=s2n,n2s=n2s)
            parent_orig_obj_path_path = copy.copy(parent_path)
            parent_orig_obj_path_path.append('orig_obj_path')
            add_dict_tree_entry(hdict,parent_orig_obj_path_path,parent['orig_obj_path'],s2n=s2n,n2s=n2s)
            parent_tag_path = copy.copy(parent_path)
            parent_tag_path.append('tag')
            add_dict_tree_entry(hdict,parent_tag_path,parent['tag'],s2n=s2n,n2s=n2s)
            parent_attrib_path = copy.copy(parent_path)
            parent_attrib_path.append('attrib')
            parent_attrib = {}
            parent_attrib['type'] = utils.get_obj_type_name(parent['node'])
            add_dict_tree_entry(hdict,parent_attrib_path,parent_attrib,s2n=s2n,n2s=n2s)
            parent_depth_path = copy.copy(parent_path)
            parent_depth_path.append('depth')
            add_dict_tree_entry(hdict,parent_depth_path,unhandled[i]['depth'],s2n=s2n,n2s=n2s)
            parent_text_path = copy.copy(parent_path)
            parent_text_path.append('text')
            if(utils.is_recursive_type(unhandled[i]['node']) | utils.is_set(unhandled[i]['node'])):
                text_temp = None
            else:
                text_temp = unhandled[i]['node']
            add_dict_tree_entry(hdict,parent_text_path,text_temp,s2n=s2n,n2s=n2s)
            parent_siblings_seq_path = copy.copy(parent_path)
            parent_siblings_seq_path.append('siblings_seq')
            add_dict_tree_entry(hdict,parent_siblings_seq_path,unhandled[i]['siblings_seq'],s2n=s2n,n2s=n2s)
            parent_breadth_path = copy.copy(parent_path)
            parent_breadth_path.append('breadth')
            add_dict_tree_entry(hdict,parent_breadth_path,unhandled[i]['breadth'],s2n=s2n,n2s=n2s)
            #
            parent_bps_path = copy.copy(parent_path)
            parent_bps_path.append('breadth_path')
            add_dict_tree_entry(hdict,parent_bps_path,unhandled[i]['breadth_path'],s2n=s2n,n2s=n2s)
            #
            parent_children_path = copy.copy(parent_path)
            parent_children_path.append('children')
            utils.dict_setdefault_via_path_list(hdict,parent_children_path)
            children = parent['node']
            if(debug):
                print("children:{0}:  {1}".format(i,children))
            else:
                pass
            if(get_sdict):
                siblings_seq_path = hdict_path_to_siblings_seq_path(parent_path)
                sdict[depth][unhandled[i]['breadth']] = {}
                sdict[depth][unhandled[i]['breadth']]['text'] = text_temp
                sdict[depth][unhandled[i]['breadth']]['attrib'] = parent_attrib
                sdict[depth][unhandled[i]['breadth']]['breadth_path'] = parent['breadth_path']
                sdict[depth][unhandled[i]['breadth']]['hdict_path'] = parent_path
                sdict[depth][unhandled[i]['breadth']]['siblings_seq_path'] = siblings_seq_path
                sdict[depth][unhandled[i]['breadth']]['orig_obj_path'] = parent['orig_obj_path']
                if(utils.is_recursive_type(children)|utils.is_set(children)):
                    sdict[depth][unhandled[i]['breadth']]['leaf'] = False
                    sdict[depth][unhandled[i]['breadth']]['html_tag_lines'] = 2
                else:
                    sdict[depth][unhandled[i]['breadth']]['leaf'] = True
                    sdict[depth][unhandled[i]['breadth']]['html_tag_lines'] = 3
                if('type' in parent_attrib):
                    sdict[depth][unhandled[i]['breadth']]['type'] = parent_attrib['type']
                else:
                    sdict[depth][unhandled[i]['breadth']]['type'] = None
            else:
                pass
            if(utils.is_list(children)|utils.is_tuple(children)):
                for j in range(0,children.__len__()):
                    child = children[j]
                    breadth = next_unhandled.__len__()
                    siblings_seq = j
                    parent_path = copy.copy(unhandled[i]['path'])
                    parent_path.append('children')
                    parent_path.append(siblings_seq)
                    path = parent_path
                    parent_orig_obj_path = copy.copy(parent['orig_obj_path'])
                    parent_orig_obj_path.append(siblings_seq)
                    parent_bps_path = copy.copy(parent['breadth_path'])
                    parent_bps_path.append(breadth)
                    next_unhandled.append({'node':child,'path':path,'tag':siblings_seq,'siblings_seq':siblings_seq,'breadth':breadth,'depth':(depth+1),'orig_obj_path':parent_orig_obj_path,'breadth_path':parent_bps_path})
            elif(utils.is_set(children)):
                temp_seq = 0
                for ele in children:
                    child = ele
                    breadth = next_unhandled.__len__()
                    siblings_seq = temp_seq
                    parent_path = copy.copy(unhandled[i]['path'])
                    parent_path.append('children')
                    parent_path.append(siblings_seq)
                    path = parent_path
                    parent_orig_obj_path = copy.copy(parent['orig_obj_path'])
                    parent_orig_obj_path_loose = parent_orig_obj_path
                    parent_orig_obj_path.append(siblings_seq)
                    parent_bps_path = copy.copy(parent['breadth_path'])
                    parent_bps_path.append(breadth)
                    next_unhandled.append({'node':child,'path':path,'tag':siblings_seq,'siblings_seq':siblings_seq,'breadth':breadth,'depth':(depth+1),'orig_obj_path':parent_orig_obj_path,'breadth_path':parent_bps_path,'orig_obj_path_loose':parent_orig_obj_path_loose})
                    temp_seq = temp_seq + 1
            elif(utils.is_dict(children)):
                seq = 0
                for key in children:
                    child = children[key]
                    breadth = next_unhandled.__len__()
                    siblings_seq = seq
                    parent_path = copy.copy(unhandled[i]['path'])
                    parent_path.append('children')
                    parent_path.append(siblings_seq)
                    path = parent_path
                    parent_orig_obj_path = copy.copy(parent['orig_obj_path'])
                    parent_orig_obj_path.append(key)
                    parent_bps_path = copy.copy(parent['breadth_path'])
                    parent_bps_path.append(breadth)
                    next_unhandled.append({'node':child,'path':path,'tag':key,'siblings_seq':siblings_seq,'breadth':breadth,'depth':(depth+1),'orig_obj_path':parent_orig_obj_path,'breadth_path':parent_bps_path})
                    seq = seq + 1
            else:
                pass
        unhandled = next_unhandled
        depth = depth + 1
    next_unhandled = None
    unhandled = None
    #now sdict is in internal state
    if(js):
        hdict_str = json.dumps(hdict,sort_keys=True,indent=4)
        #json.dumps will convert number key to str key 
        #json.dumps({0:'a',1:'b'})
        #'{"0": "a", "1": "b"}'
        hdict = json.loads(hdict_str)
        # json.loads Expecting property name enclosed in double quotes:
        # json.loads('{0:"a",1:"b"}') ERROR
        # json.loads('{"0":"a","1":"b"}') OK 
    else:
        hdict_str = hdict.__str__()
    if(save):
        fd = open(kwargs['fn'],kwargs['op'])
        fd.write(hdict_str)
        fd.close()
    else:
        pass
    prdict = hdict_get_paths_relationship(hdict)
    #get final sdict
    sdict = hdict_fullfill_sdict(sdict,hdict,prdict)
    return({'hdict':hdict,'sdict':sdict,'prdict':prdict})

def hdict_get_paths_relationship(hdict):
    r = {}
    r['h:b'] = {():[]}
    r['b:h'] = {():[]}
    r['o:h'] = {():[]}
    r['h:o'] = {():[]}
    unhandled = []
    for i in range(0,hdict.__len__()):
        unhandled.append(hdict[i])
    while(unhandled.__len__()>0):
        next_unhandled = []
        for i in range(0,unhandled.__len__()):
            e = unhandled[i]
            h = e['path']
            b = e['breadth_path']
            o = e['orig_obj_path']
            try:
                t = e['attrib']['type']
            except:
                t = None
            else:
                pass
            r['h:b'][tuple(h)] = b
            r['b:h'][tuple(b)] = h
            r['h:o'][tuple(h)] = o
            r['o:h'][tuple(o)] = h
            if(e['children'].__len__() == 0):
                pass
            else:
                child = e['children']
                for j in range(0,child.__len__()):
                    next_unhandled.append(child[j])
        unhandled = next_unhandled
    return(r)

#----------------obdseleted-----------------------
def obseleted_hdict_fullfill_sdict(sdict,hdict,prdict):
    for i in range(sdict.__len__()-1,-1,-1):
        r = sdict[i]
        for j in range(r.__len__()-1,-1,-1):
            d = r[j]
            d['leaf_sons'] = 0
            d['non_leaf_sons']= 0
            d['leaf_descendants']= 0
            d['non_leaf_descendants']= 0
    for i in range(sdict.__len__()-1,-1,-1):
        r = sdict[i]
        for j in range(r.__len__()-1,-1,-1):
            d = r[j]
            if(d['breadth_path'].__len__()>1):
                parent_breadth = d['breadth_path'][-2]
                if('leaf_sons' in sdict[i-1][parent_breadth]):
                    if(d['leaf']):
                        sdict[i-1][parent_breadth]['leaf_sons'] = sdict[i-1][parent_breadth]['leaf_sons'] + 1
                    else:
                        pass
                else:
                    if(d['leaf']):
                        sdict[i-1][parent_breadth]['leaf_sons'] = 1
                    else:
                        pass
                if('non_leaf_sons' in sdict[i-1][parent_breadth]):
                    if(d['leaf']):
                        pass
                    else:
                        sdict[i-1][parent_breadth]['non_leaf_sons'] = sdict[i-1][parent_breadth]['non_leaf_sons'] + 1
                else:
                    if(d['leaf']):
                        pass
                    else:
                        sdict[i-1][parent_breadth]['non_leaf_sons'] = 1
                if('leaf_descendants' in sdict[i-1][parent_breadth]):
                    if(d['leaf']):
                        sdict[i-1][parent_breadth]['leaf_descendants'] = sdict[i-1][parent_breadth]['leaf_descendants']+d['leaf_descendants'] + 1
                    else:
                        sdict[i-1][parent_breadth]['leaf_descendants'] = sdict[i-1][parent_breadth]['leaf_descendants']+d['leaf_descendants']
                else:
                    if(d['leaf']):
                        sdict[i-1][parent_breadth]['leaf_descendants'] = d['leaf_descendants'] + 1
                    else:
                        sdict[i-1][parent_breadth]['leaf_descendants'] = d['leaf_descendants']
                if('non_leaf_descendants' in sdict[i-1][parent_breadth]):
                    if(d['leaf']):
                        sdict[i-1][parent_breadth]['non_leaf_descendants'] = sdict[i-1][parent_breadth]['non_leaf_descendants']+d['non_leaf_descendants'] 
                    else:
                        sdict[i-1][parent_breadth]['non_leaf_descendants'] = sdict[i-1][parent_breadth]['non_leaf_descendants']+d['non_leaf_descendants'] + 1
                else:
                    if(d['leaf']):
                        sdict[i-1][parent_breadth]['non_leaf_descendants'] = d['non_leaf_descendants'] 
                    else:
                        sdict[i-1][parent_breadth]['non_leaf_descendants'] = d['non_leaf_descendants'] + 1
            else:
                pass
    for i in range(sdict.__len__()-1,-1,-1):
        r = sdict[i]
        for j in range(r.__len__()-1,-1,-1):
            d = r[j]
            d['hdict_parent_path'] = copy.copy(d['hdict_path'])
            d['hdict_parent_path'].pop(-1)
            d['orig_parent_path'] = copy.copy(d['orig_obj_path'])
            d['orig_parent_path'].pop(-1)
            d['hdict_lsib_path'] = hdict_lsib_path(hdict,sdict,prdict,d['hdict_path'])
            d['orig_lsib_path'] = hdict_path_to_orig_obj_path(prdict,d['hdict_lsib_path'])
            d['hdict_rsib_path'] = hdict_rsib_path(hdict,sdict,prdict,d['hdict_path'])
            d['orig_rsib_path'] = hdict_path_to_orig_obj_path(prdict,d['hdict_rsib_path'])
            d['hdict_lcin_path'] = hdict_lcin_path(hdict,sdict,prdict,d['hdict_path'])
            d['orig_lcin_path'] = hdict_path_to_orig_obj_path(prdict,d['hdict_lcin_path'])
            d['hdict_rcin_path'] = hdict_rcin_path(hdict,sdict,prdict,d['hdict_path'])
            d['orig_rcin_path'] = hdict_path_to_orig_obj_path(prdict,d['hdict_rcin_path'])
    return(sdict)

#--------------------------------------------------------



#---------------------

def hdict_fullfill_sdict(sdict,hdict,prdict):
    for i in range(sdict.__len__()-1,-1,-1):
        r = sdict[i]
        for j in range(r.__len__()-1,-1,-1):
            d = r[j]
            d['leaf_sons'] = 0
            d['non_leaf_sons']= 0
            d['leaf_descendants']= 0
            d['non_leaf_descendants']= 0
            #################################
            d['leaf_son_pathlists'] = []
            d['non_leaf_son_pathlists']= []
            d['leaf_descendant_pathlists']= []
            d['non_leaf_descendant_pathlists']= []
            ##################################
    for i in range(sdict.__len__()-1,-1,-1):
        r = sdict[i]
        for j in range(r.__len__()-1,-1,-1):
            d = r[j]
            if(d['breadth_path'].__len__()>1):
                parent_breadth = d['breadth_path'][-2]
                if('leaf_sons' in sdict[i-1][parent_breadth]):
                    if(d['leaf']):
                        sdict[i-1][parent_breadth]['leaf_sons'] = sdict[i-1][parent_breadth]['leaf_sons'] + 1
                        ########################
                        sdict[i-1][parent_breadth]['leaf_son_pathlists'].append(d['orig_obj_path'])
                        ########################
                    else:
                        pass
                else:
                    if(d['leaf']):
                        sdict[i-1][parent_breadth]['leaf_sons'] = 1
                        ########################
                        sdict[i-1][parent_breadth]['leaf_son_pathlists'].append(d['orig_obj_path'])
                        ########################
                    else:
                        pass
                if('non_leaf_sons' in sdict[i-1][parent_breadth]):
                    if(d['leaf']):
                        pass
                    else:
                        sdict[i-1][parent_breadth]['non_leaf_sons'] = sdict[i-1][parent_breadth]['non_leaf_sons'] + 1
                        ########################
                        sdict[i-1][parent_breadth]['non_leaf_son_pathlists'].append(d['orig_obj_path'])
                        ########################
                else:
                    if(d['leaf']):
                        pass
                    else:
                        sdict[i-1][parent_breadth]['non_leaf_sons'] = 1
                        ########################
                        sdict[i-1][parent_breadth]['non_leaf_son_pathlists'].append(d['orig_obj_path'])
                        ########################
                if('leaf_descendants' in sdict[i-1][parent_breadth]):
                    if(d['leaf']):
                        sdict[i-1][parent_breadth]['leaf_descendants'] = sdict[i-1][parent_breadth]['leaf_descendants']+d['leaf_descendants'] + 1
                        ##################################
                        sdict[i-1][parent_breadth]['leaf_descendant_pathlists'] = sdict[i-1][parent_breadth]['leaf_descendant_pathlists']+d['leaf_descendant_pathlists']
                        sdict[i-1][parent_breadth]['leaf_descendant_pathlists'].append(d['orig_obj_path'])
                        ##################################
                    else:
                        sdict[i-1][parent_breadth]['leaf_descendants'] = sdict[i-1][parent_breadth]['leaf_descendants']+d['leaf_descendants']
                        ##################################
                        sdict[i-1][parent_breadth]['leaf_descendant_pathlists'] = sdict[i-1][parent_breadth]['leaf_descendant_pathlists']+d['leaf_descendant_pathlists']
                        ###############################@@##
                        ###############################@@##
                else:
                    if(d['leaf']):
                        sdict[i-1][parent_breadth]['leaf_descendants'] = d['leaf_descendants'] + 1
                        ##################################
                        sdict[i-1][parent_breadth]['leaf_descendant_pathlists'] = d['leaf_descendant_pathlistss']
                        sdict[i-1][parent_breadth]['leaf_descendant_pathlists'].append(d['orig_obj_path'])
                        ##################################
                    else:
                        sdict[i-1][parent_breadth]['leaf_descendants'] = d['leaf_descendants']
                        ##################################
                        sdict[i-1][parent_breadth]['leaf_descendant_pathlists'] = d['leaf_descendant_pathlistss']
                        ##################################
                if('non_leaf_descendants' in sdict[i-1][parent_breadth]):
                    if(d['leaf']):
                        pass
                        ##################################
                        ##################################
                    else:
                        sdict[i-1][parent_breadth]['non_leaf_descendants'] = sdict[i-1][parent_breadth]['non_leaf_descendants']+d['non_leaf_descendants'] + 1
                        ##################################
                        sdict[i-1][parent_breadth]['non_leaf_descendant_pathlists'] =  sdict[i-1][parent_breadth]['non_leaf_descendant_pathlists']+d['non_leaf_descendant_pathlists']
                        sdict[i-1][parent_breadth]['non_leaf_descendant_pathlists'].append(d['orig_obj_path'])
                        ##################################
                else:
                    if(d['leaf']):
                        pass
                        ##################################
                    else:
                        sdict[i-1][parent_breadth]['non_leaf_descendants'] = d['non_leaf_descendants'] + 1
                        ##################################
                        sdict[i-1][parent_breadth]['non_leaf_descendant_pathlists'] = d['non_leaf_descendant_pathlists']
                        sdict[i-1][parent_breadth]['non_leaf_descendant_pathlists'].append(d['orig_obj_path'])
                        ##################################
            else:
                pass
    for i in range(sdict.__len__()-1,-1,-1):
        r = sdict[i]
        for j in range(r.__len__()-1,-1,-1):
            d = r[j]
            d['hdict_parent_path'] = copy.copy(d['hdict_path'])
            d['hdict_parent_path'].pop(-1)
            d['orig_parent_path'] = copy.copy(d['orig_obj_path'])
            d['orig_parent_path'].pop(-1)
            d['hdict_lsib_path'] = hdict_lsib_path(hdict,sdict,prdict,d['hdict_path'])
            d['orig_lsib_path'] = hdict_path_to_orig_obj_path(prdict,d['hdict_lsib_path'])
            d['hdict_rsib_path'] = hdict_rsib_path(hdict,sdict,prdict,d['hdict_path'])
            d['orig_rsib_path'] = hdict_path_to_orig_obj_path(prdict,d['hdict_rsib_path'])
            d['hdict_lcin_path'] = hdict_lcin_path(hdict,sdict,prdict,d['hdict_path'])
            d['orig_lcin_path'] = hdict_path_to_orig_obj_path(prdict,d['hdict_lcin_path'])
            d['hdict_rcin_path'] = hdict_rcin_path(hdict,sdict,prdict,d['hdict_path'])
            d['orig_rcin_path'] = hdict_path_to_orig_obj_path(prdict,d['hdict_rcin_path'])
    return(sdict)












#--------------------




def hdict_get_value(hdict,path_list_or_path_string,**kwargs):
    if('prdict' in kwargs):
        prdict = kwargs['prdict']
    else:
        prdict = hdict_get_paths_relationship(hdict)
    if('method' in kwargs):
        method = kwargs['method']
    else:
        method = 'orig_obj_path'
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    if(utils.is_list(path_list_or_path_string)):
        pl = path_list_or_path_string
    else:
        pl = path_list_or_path_string.split(sp)
    if(method == 'hdict_path'):
        pass
    if(method == 'siblings_seq_path'):
        pl = siblings_seq_path_to_hdict_path(pl,sp=sp)
    if(method == 'breadth_path'):
        pl = breadth_path_to_hdict_path(prdict,pl,sp=sp)
    if(method == 'orig_obj_path'):
        pl = orig_obj_path_to_hdict_path(prdict,pl,sp=sp)
    rslt = utils.dict_getitem_via_path_list(hdict,pl,s2n=1)
    return(rslt)

def get_sdict_and_prdict_from_hdict(hdict,**kwargs):
    if('disable_type' in kwargs):
        disable_type = kwargs['disable_type']
    else:
        disable_type = 0
    sdict = {}
    unhandled = []
    depth = 0
    for i in range(0,hdict.__len__()):
        unhandled.append(hdict[i])
    while(unhandled.__len__()>0):
        sdict[depth] = {}
        next_unhandled = []
        for i in range(0,unhandled.__len__()):
            e = unhandled[i]
            sdict[depth][e['breadth']] = {}
            sdict[depth][e['breadth']]['text'] = e['text']
            sdict[depth][e['breadth']]['breadth_path'] = e['breadth_path']
            sdict[depth][e['breadth']]['hdict_path'] = e['path']
            sdict[depth][e['breadth']]['siblings_seq_path'] = hdict_path_to_siblings_seq_path(e['path'])
            sdict[depth][e['breadth']]['orig_obj_path'] = e['orig_obj_path']
            #
            sdict[depth][e['breadth']]['attrib'] = e['attrib']
            #
            if(disable_type):
                sdict[depth][e['breadth']]['type'] = None
            else:
                if('type' in e['attrib']):
                    sdict[depth][e['breadth']]['type'] = e['attrib']['type']
                else:
                    sdict[depth][e['breadth']]['type'] = None
            if(e['children'].__len__() == 0):
                sdict[depth][e['breadth']]['leaf'] = True
                sdict[depth][e['breadth']]['html_tag_lines'] = 3
            else:
                sdict[depth][e['breadth']]['leaf'] = False
                sdict[depth][e['breadth']]['html_tag_lines'] = 2
                children = e['children']
                for j in range(0,children.__len__()):
                    next_unhandled.append(children[j])
        unhandled = next_unhandled
        depth = depth + 1
    prdict = hdict_get_paths_relationship(hdict)
    #now sdict is internal state
    sdict = hdict_fullfill_sdict(sdict,hdict,prdict)
    #final state sdict
    return({'sdict':sdict,'prdict':prdict})

def get_path_list(path_list_or_path_string,sp):
    if(ltdict.is_ltdict(path_list_or_path_string)):
        path_list = ltdict.ltdict2list(path_list_or_path_string)
    elif(utils.is_list(path_list_or_path_string)):
        path_list = path_list_or_path_string
    else:
        path_list = path_list_or_path_string.split(sp)
    return(path_list)

def hdict_path_to_siblings_seq_path(path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    siblings_seq_path = []
    for i in range(0,path_list.__len__()):
        if(path_list[i] == 'children'):
            pass
        else:
            siblings_seq_path.append(path_list[i])
    return(siblings_seq_path)

def siblings_seq_path_to_hdict_path(path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    hdict_path = []
    for i in range(0,path_list.__len__()):
        hdict_path.append(path_list[i])
        hdict_path.append('children')
    hdict_path.pop(-1)
    return(hdict_path)

def hdict_path_to_breadth_path(hdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    temp = utils.dict_getitem_via_path_list(hdict,path_list,s2n=1)
    breadth_path = copy.copy(temp['breadth_path'])
    return(breadth_path)

def breadth_path_to_hdict_path(prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    hdict_path = prdict['b:h'][tuple(path_list)]
    return(hdict_path)

def orig_obj_path_to_hdict_path(prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    hdict_path = prdict['o:h'][tuple(path_list)]
    return(hdict_path)

def hdict_path_to_orig_obj_path(prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    orig_obj_path = prdict['h:o'][tuple(path_list)]
    return(orig_obj_path)

def breadth_path_to_sdict_location(path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    r = path_list.__len__() - 1
    if(r<0):
        c = -1
    else:
        c = path_list[-1]
    return(r,c)

def get_siblings_number_from_breadth_path(sdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    breadth_path = get_path_list(path_list_or_path_string,sp)
    parent_breadth_path = copy.copy(breadth_path)
    parent_breadth_path.pop(-1)
    if(parent_breadth_path.__len__()==0):
        siblings_number = 0
    else:
        r,c = breadth_path_to_sdict_location(parent_breadth_path,sp=sp)
        siblings_number = sdict[r][c]['leaf_sons'] + sdict[r][c]['non_leaf_sons']
    return(siblings_number)

def get_cousins_number_from_breadth_path(sdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    breadth_path = get_path_list(path_list_or_path_string,sp)
    depth = breadth_path.__len__() - 1
    cousins_number = sdict[depth].__len__()
    return(cousins_number)

def hdict_parent_path(hdict,pl,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl = get_path_list(pl,sp)
    pl = copy.copy(p1)
    p1 = p1[:-2]
    return(pl)

def hdict_lsib_path(hdict,sdict,prdict,path_list_or_path_string,**kwargs): 
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    siblings_seq_path = hdict_path_to_siblings_seq_path(path_list,sp=sp)
    siblings_seq_path[-1] = siblings_seq_path[-1] - 1
    lsib_siblings_seq_path = siblings_seq_path
    if(lsib_siblings_seq_path[-1]<0):
        return([])
    else:
        lsib_hdict_path = siblings_seq_path_to_hdict_path(lsib_siblings_seq_path,sp=sp)
        return(lsib_hdict_path)

def hdict_rsib_path(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    breadth_path = hdict_path_to_breadth_path(hdict,path_list,sp=sp)
    max_siblings_seq = get_siblings_number_from_breadth_path(sdict,breadth_path,sp=sp) - 1
    siblings_seq_path = hdict_path_to_siblings_seq_path(path_list,sp=sp)
    siblings_seq_path[-1] = siblings_seq_path[-1] + 1
    rsib_siblings_seq_path = siblings_seq_path
    if(rsib_siblings_seq_path[-1]>max_siblings_seq):
        return([])
    else:
        rsib_hdict_path = siblings_seq_path_to_hdict_path(rsib_siblings_seq_path,sp=sp)
        return(rsib_hdict_path)

def hdict_lcin_path(hdict,sdict,prdict,path_list_or_path_string,**kwargs): 
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    breadth_path = hdict_path_to_breadth_path(hdict,path_list,sp=sp)
    max_cousin_seq = get_cousins_number_from_breadth_path(sdict,breadth_path,sp=sp)
    r = breadth_path.__len__() - 1
    c = breadth_path[-1] - 1
    if(c<0):
        return([])
    else:
        siblings_seq_path = hdict_path_to_siblings_seq_path(path_list,sp=sp)
        siblings_seq_path[-1] = siblings_seq_path[-1] - 1
        lsib_siblings_seq_path = siblings_seq_path
        if(lsib_siblings_seq_path[-1]<0):
            lcin_breadth_path = sdict[r][c]['breadth_path']
            lcin_hdict_path = breadth_path_to_hdict_path(prdict,lcin_breadth_path,sp=sp)
        else:
            return([])
        return(lcin_hdict_path)

def hdict_rcin_path(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    breadth_path = hdict_path_to_breadth_path(hdict,path_list,sp=sp)
    max_cousin_seq = get_cousins_number_from_breadth_path(sdict,breadth_path,sp=sp) - 1
    max_siblings_seq = get_siblings_number_from_breadth_path(sdict,breadth_path,sp=sp) - 1
    r = breadth_path.__len__() - 1
    c = breadth_path[-1] + 1
    if(c>max_cousin_seq):
        return([])
    else:
        siblings_seq_path = hdict_path_to_siblings_seq_path(path_list,sp=sp)
        siblings_seq_path[-1] = siblings_seq_path[-1] + 1
        rsib_siblings_seq_path = siblings_seq_path
        if(rsib_siblings_seq_path[-1]>max_siblings_seq):
            rcin_breadth_path = sdict[r][c]['breadth_path']
            rcin_hdict_path = breadth_path_to_hdict_path(prdict,rcin_breadth_path,sp=sp)
        else:
            return([])
    return(rcin_hdict_path)

def orig_parent_path(hdict,pl,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl = get_path_list(pl,sp)
    pl = copy.copy(p1)
    p1 = p1[:-1]
    return(pl)

def orig_lsib_path(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol = get_path_list(path_list_or_path_string,sp)
    pl = orig_obj_path_to_hdict_path(prdict,ol,sp=sp)
    lsib_hdict_path = hdict_lsib_path(hdict,sdict,prdict,pl,sp=sp)
    orig_lsib_path = hdict_path_to_orig_obj_path(prdict,lsib_hdict_path,sp=sp)
    return(orig_lsib_path)

def orig_rsib_path(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol = get_path_list(path_list_or_path_string,sp)
    pl = orig_obj_path_to_hdict_path(prdict,ol,sp=sp)
    rsib_hdict_path = hdict_rsib_path(hdict,sdict,prdict,pl,sp=sp)
    orig_rsib_path = hdict_path_to_orig_obj_path(prdict,rsib_hdict_path,sp=sp)
    return(orig_rsib_path)

def orig_lcin_path(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol = get_path_list(path_list_or_path_string,sp)
    pl = orig_obj_path_to_hdict_path(prdict,ol,sp=sp)
    lcin_hdict_path = hdict_lcin_path(hdict,sdict,prdict,pl,sp=sp)
    orig_lcin_path = hdict_path_to_orig_obj_path(prdict,lcin_hdict_path,sp=sp)
    return(orig_lcin_path)

def orig_rcin_path(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol = get_path_list(path_list_or_path_string,sp)
    pl = orig_obj_path_to_hdict_path(prdict,ol,sp=sp)
    rcin_hdict_path = hdict_rcin_path(hdict,sdict,prdict,pl,sp=sp)
    orig_rcin_path = hdict_path_to_orig_obj_path(prdict,rcin_hdict_path,sp=sp)
    return(orig_rcin_path)

def hdict_relationship(pl1,pl2,hdict,sdict,prdict,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl1 = get_path_list(pl1,sp)
    pl1 = get_path_list(pl2,sp)
    bp1 = hdict_path_to_breadth_path(hdict,pl1,sp=sp)
    bp2 = hdict_path_to_breadth_path(hdict,pl2,sp=sp)
    len1 = bp1.__len__()
    len2 = bp2.__len__()
    i = 0
    while((i<len1) & (i<len2)):
        if(bp1[i] == bp2[i]):
            i = i + 1
        else:
            break
    if((i==len1)|(i==len2)):
        nearest_ancestor = []
    else:
        nearest_ancestor = []
        for j in range(0,i):
            nearest_ancestor.append(bp1[j])
    dist = len1 - nearest_ancestor.__len__() + len2 - nearest_ancestor.__len__()
    nearest_ancestor = breadth_path_to_hdict_path(prdict,nearest_ancestor,sp=sp)
    l = nearest_ancestor.__len__()
    ancestor_to_x = copy.copy(pl1)[i:]
    ancestor_to_y = copy.copy(pl2)[i:]
    rel = {}
    rel['dist'] = dist
    rel['nearest_ancestor'] = nearest_ancestor
    rel['ancestor_to_x'] = ancestor_to_x
    rel['ancestor_to_y'] = ancestor_to_y
    return(rel)

def hdict_is_ancestor(pl1,pl2):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl1 = get_path_list(pl1,sp)
    pl2 = get_path_list(pl2,sp)
    pl1 = hdict_path_to_breadth_path(pl1)
    pl2 = hdict_path_to_breadth_path(pl2)
    if(pl1.__len__()<pl2.__len__()):
        if(pl1[:pl1.__len__()] == pl2[:pl1.__len__()]):
            return(True)
        else:
            return(False)
    else:
        return(False)

def hdict_is_parent(pl1,pl2):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl1 = get_path_list(pl1,sp)
    pl2 = get_path_list(pl2,sp)
    pl1 = hdict_path_to_breadth_path(pl1)
    pl2 = hdict_path_to_breadth_path(pl2)
    if(pl1.__len__()==(pl2.__len__()-1)):
        if(pl1[:pl1.__len__()] == pl2[:pl1.__len__()]):
            return(True)
        else:
            return(False)
    else:
        return(False)

def hdict_is_decesdant(pl1,pl2):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl1 = get_path_list(pl1,sp)
    pl2 = get_path_list(pl2,sp)
    pl1 = hdict_path_to_breadth_path(pl1)
    pl2 = hdict_path_to_breadth_path(pl2)
    if(pl1.__len__()>pl2.__len__()):
        if(pl1[:pl2.__len__()] == pl2[:pl2.__len__()]):
            return(True)
        else:
            return(False)
    else:
        return(False)

def hdict_is_son(pl1,pl2):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl1 = get_path_list(pl1,sp)
    pl2 = get_path_list(pl2,sp)
    pl1 = hdict_path_to_breadth_path(pl1)
    pl2 = hdict_path_to_breadth_path(pl2)
    if(pl1.__len__()==(pl2.__len__()+1)):
        if(pl1[:pl2.__len__()] == pl2[:pl2.__len__()]):
            return(True)
        else:
            return(False)
    else:
        return(False)

def hdict_is_lsib(pl1,pl2,hdict,sdict,prdict,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl1 = get_path_list(pl1,sp)
    pl2 = get_path_list(pl2,sp)
    lsibp = hdict_lsib_path(hdict,sdict,prdict,pl2,sp=sp)
    if(lsibp == pl1):
        return(True)
    else:
        return(False)

def hdict_is_rsib(pl1,pl2,hdict,sdict,prdict,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl1 = get_path_list(pl1,sp)
    pl2 = get_path_list(pl2,sp)
    rsibp = hdict_rsib_path(hdict,sdict,prdict,pl2,sp=sp)
    if(rsibp == pl1):
        return(True)
    else:
        return(False)

def hdict_is_lcin(pl1,pl2,hdict,sdict,prdict,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl1 = get_path_list(pl1,sp)
    pl2 = get_path_list(pl2,sp)
    lcinp = hdict_lcin_path(hdict,sdict,prdict,pl2,sp=sp)
    if(lcinp == pl1):
        return(True)
    else:
        return(False)

def hdict_is_rcin(pl1,pl2,hdict,sdict,prdict,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    pl1 = get_path_list(pl1,sp)
    pl2 = get_path_list(pl2,sp)
    rcinp = hdict_rcin_path(hdict,sdict,prdict,pl2,sp=sp)
    if(rcinp == pl1):
        return(True)
    else:
        return(False)

def orig_relationship(ol1,ol2,hdict,sdict,prdict,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    len1 = ol1.__len__()
    len2 = ol2.__len__()
    i = 0
    while((i<len1) & (i<len2)):
        if(ol1[i] == ol2[i]):
            i = i + 1
        else:
            break
    if((i==len1)|(i==len2)):
        nearest_ancestor = []
    else:
        nearest_ancestor = []
        for j in range(0,i):
            nearest_ancestor.append(ol1[j])
    dist = len1 - nearest_ancestor.__len__() + len2 - nearest_ancestor.__len__()
    l = nearest_ancestor.__len__()
    ancestor_to_x = copy.copy(ol1)[i:]
    ancestor_to_y = copy.copy(ol2)[i:]
    rel = {}
    rel['dist'] = dist
    rel['nearest_ancestor'] = nearest_ancestor
    rel['ancestor_to_x'] = ancestor_to_x
    rel['ancestor_to_y'] = ancestor_to_y
    return(rel)

def orig_is_ancestor(ol1,ol2,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    if(ol1.__len__()<ol2.__len__()):
        if(ol1[:ol1.__len__()] == ol2[:ol1.__len__()]):
            return(True)
        else:
            return(False)
    else:
        return(False)

def orig_is_parent(ol1,ol2,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    if(ol1.__len__()==(ol2.__len__()-1)):
        if(ol1[:ol1.__len__()] == ol2[:ol1.__len__()]):
            return(True)
        else:
            return(False)
    else:
        return(False)

def orig_is_decesdant(ol1,ol2,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    if(ol1.__len__()>ol2.__len__()):
        if(ol1[:ol2.__len__()] == ol2[:ol2.__len__()]):
            return(True)
        else:
            return(False)
    else:
        return(False)

def orig_is_son(ol1,ol2,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    if(ol1.__len__()==(ol2.__len__()+1)):
        if(ol1[:ol2.__len__()] == ol2[:ol2.__len__()]):
            return(True)
        else:
            return(False)
    else:
        return(False)

def orig_is_lsib(ol1,ol2,hdict,sdict,prdict,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol1 = get_path_list(ol1,sp)
    ol2 = get_path_list(ol2,sp)
    lsibp = orig_lsib_path(hdict,sdict,prdict,ol2,sp=sp)
    if(lsibp == ol1):
        return(True)
    else:
        return(False)

def orig_is_rsib(ol1,ol2,hdict,sdict,prdict,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol1 = get_path_list(ol1,sp)
    ol2 = get_path_list(ol2,sp)
    rsibp = orig_rsib_path(hdict,sdict,prdict,ol2,sp=sp)
    if(rsibp == ol1):
        return(True)
    else:
        return(False)

def orig_is_lcin(ol1,ol2,hdict,sdict,prdict,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol1 = get_path_list(ol1,sp)
    ol2 = get_path_list(ol2,sp)
    lcinp = orig_lcin_path(hdict,sdict,prdict,ol2,sp=sp)
    if(lcinp == ol1):
        return(True)
    else:
        return(False)

def orig_is_rcin(ol1,ol2,hdict,sdict,prdict,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol1 = get_path_list(ol1,sp)
    ol2 = get_path_list(ol2,sp)
    rcinp = orig_rcin_path(hdict,sdict,prdict,ol2,sp=sp)
    if(rcinp == ol1):
        return(True)
    else:
        return(False)

def hdict_parent(hdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    ppl = hdict_parent_path(hdict,path_list,sp=sp)
    rslt = hdict_get_value(hdict,ppl,method='hdict_path')
    return(rslt)

def hdict_lsib(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    lsibpl = hidct_lsib_path(hdict,sdict,prdict,path_list,sp=sp)
    rslt = hdict_get_value(hdict,lsibpl,method='hdict_path')
    return(rslt)

def hdict_rsib(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    rsibpl = hidct_rsib_path(hdict,sdict,prdict,path_list,sp=sp)
    rslt = hdict_get_value(hdict,rsibpl,method='hdict_path')
    return(rslt)

def hdict_lcin(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    lcinpl = hidct_lcin_path(hdict,sdict,prdict,path_list,sp=sp)
    rslt = hdict_get_value(hdict,lcinpl,method='hdict_path')
    return(rslt)

def hdict_rcin(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    path_list = get_path_list(path_list_or_path_string,sp)
    rcinpl = hidct_rcin_path(hdict,sdict,prdict,path_list,sp=sp)
    rslt = hdict_get_value(hdict,rcinpl,method='hdict_path')
    return(rslt)

def orig_parent(obj,hdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol = get_path_list(path_list_or_path_string,sp)
    pl = orig_obj_path_to_hdict_path(prdict,ol,sp=sp)
    ppl = hidct_parent_path(hdict,pl,sp=sp)
    pol = hdict_path_to_orig_obj_path(prdict,ppl,sp=sp)
    rslt = utils.dict_getitem_via_path_list(obj,pol,s2n=1)
    return(rslt)

def orig_lsib(obj,hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol = get_path_list(path_list_or_path_string,sp)
    pl = orig_obj_path_to_hdict_path(prdict,ol,sp=sp)
    lsibpl = hdict_lsib_path(hdict,sdict,prdict,pl,sp=sp)
    lsibol = hdict_path_to_orig_obj_path(prdict,lsibpl,sp=sp)
    rslt = utils.dict_getitem_via_path_list(obj,lsibol,s2n=1)
    return(rslt)

def orig_rsib(hdict,sdict,prdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol = get_path_list(path_list_or_path_string,sp)
    pl = orig_obj_path_to_hdict_path(prdict,ol,sp=sp)
    rsibpl = hdict_rsib_path(hdict,sdict,prdict,pl,sp=sp)
    rsibol = hdict_path_to_orig_obj_path(prdict,rsibpl,sp=sp)
    rslt = utils.dict_getitem_via_path_list(obj,rsibol,s2n=1)
    return(rslt)

def orig_lcin(hdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol = get_path_list(path_list_or_path_string,sp)
    pl = orig_obj_path_to_hdict_path(prdict,ol,sp=sp)
    lcinpl = hdict_lcin_path(hdict,sdict,prdict,pl,sp=sp)
    lcinol = hdict_path_to_orig_obj_path(prdict,lcinpl,sp=sp)
    rslt = utils.dict_getitem_via_path_list(obj,lcinol,s2n=1)
    return(rslt)

def orig_rcin(hdict,path_list_or_path_string,**kwargs):
    if('sp' in kwargs):
        sp = kwargs['sp']
    else:
        sp = '/'
    ol = get_path_list(path_list_or_path_string,sp)
    pl = orig_obj_path_to_hdict_path(prdict,ol,sp=sp)
    rcinpl = hdict_rcin_path(hdict,sdict,prdict,pl,sp=sp)
    rcinol = hdict_path_to_orig_obj_path(prdict,rcinpl,sp=sp)
    rslt = utils.dict_getitem_via_path_list(obj,rcinol,s2n=1)
    return(rslt)

def hdict_to_obj(hdict,sdict,prdict,**kwargs):
    def add_dict_tree_entry(obj,path,value,n2s=0,s2n=0):
        utils.dict_setdefault_via_path_list(obj,path,n2s=n2s,s2n=s2n)
        utils.dict_setitem_via_path_list(obj,path,value,n2s=n2s,s2n=s2n)
    if('n2s' in kwargs):
        n2s = kwargs['n2s']
    else:
        n2s = 0
    if('s2n' in kwargs):
        s2n = kwargs['n2s']
    else:
        s2n = 0
    if('check_type' in kwargs):
        check_type = kwargs['check_type']
    else:
        check_type = 1
    if('deepcopy' in kwargs):
        deepcopy = kwargs['deepcopy']
    else:
        deepcopy = 1
    if(deepcopy):
        newh = copy.copy(hdict)
    else:
        newh = hdict
    obj = {}
    for i in range(sdict.__len__()-1,-1,-1):
        r = sdict[i]
        for j in range (0,r.__len__()):
            c = r[j]
            path = c['orig_obj_path']
            if(c['leaf']):
                if((c['type']== '') | (c['type']=='dict') ):
                    value = {}
                elif((c['type']== 'list')):
                    value = []
                elif((c['type']== 'tuple')):
                    value =()
                elif((c['type']== 'set')):
                    value = set({})
                elif((c['type']== 'str')):
                    value = utils.dict_getitem_via_path_list(newh,c['hdict_path'],n2s=n2s,s2n=s2n)
                    value = value['text']
                    value = str(value)
                elif((c['type']== 'int')):
                    value = utils.dict_getitem_via_path_list(newh,c['hdict_path'],n2s=n2s,s2n=s2n)
                    value = value['text']
                    value = int(value)
                elif((c['type']== 'float')):
                    value = utils.dict_getitem_via_path_list(newh,c['hdict_path'],n2s=n2s,s2n=s2n)
                    value = value['text']
                    value = float(value)
                else:
                    value = utils.dict_getitem_via_path_list(newh,c['hdict_path'],n2s=n2s,s2n=s2n)
                    value = value['text']
                add_dict_tree_entry(obj,path,value,n2s=n2s,s2n=s2n)
            else:
                if((c['type']== '') | (c['type']=='dict') ):
                    pass
                elif((c['type']== 'list')):
                    value = utils.dict_getitem_via_path_list(obj,path,n2s=n2s,s2n=s2n)
                    new_value = []
                    for k in range(0,value.__len__()):
                        new_value.append(value[k])
                    utils.dict_setitem_via_path_list(obj,path,new_value,n2s=n2s,s2n=s2n)
                elif((c['type']== 'tuple')):
                    value = utils.dict_getitem_via_path_list(obj,path,n2s=n2s,s2n=s2n)
                    new_value = []
                    for k in range(0,value.__len__()):
                        new_value.append(value[k])
                    new_value = tuple(new_value)
                    utils.dict_setitem_via_path_list(obj,path,new_value,n2s=n2s,s2n=s2n)
                elif((c['type']== 'set')):
                    value = utils.dict_getitem_via_path_list(obj,path,n2s=n2s,s2n=s2n)
                    new_value = []
                    for k in range(0,value.__len__()):
                        new_value.append(value[k])
                    new_value = set(new_value)
                    utils.dict_setitem_via_path_list(obj,path,new_value,n2s=n2s,s2n=s2n)
                else:
                    pass
    return(obj)

def xml_indent_prepend(path_list_or_path_string,**kwargs):
    if('fixed_indent' in kwargs):
        fixed_indent = kwargs['fixed_indent']
    else:
        fixed_indent = 0
    if('indent' in kwargs):
        indent = kwargs['indent']
    else:
        indent = 4
    if('path_sp' in kwargs):
        path_sp = kwargs['path_sp']
    else:
        path_sp = '/'
    pl = get_path_list(path_list_or_path_string,path_sp)
    if(fixed_indent):
        prepend = ' '*4*pl.__len__()
    else:
        c = 0
        for i in range(0,pl.__len__()):
           c = c + str(pl[i]).__len__()+2
        prepend = ' '*c
    return(prepend)

def attrib_dict_to_str(attrib):
    s = ''
    for key in attrib:
        if(attrib[key]):
            s = ''.join((s,' ',key,'=','"',str(attrib[key]),'"'))
        else:
            s = ''.join((s,' ',key))
    return(s)

def creat_xml_tag_line_label(sdict,**kwargs):
    if('noattrib' in kwargs):
        noattrib = kwargs['noattrib']
    else:
        noattrib = 0
    if('clear' in kwargs):
        clear = kwargs['clear']
    else:
        clear = 1
    if('deepcopy' in kwargs):
        deepcopy = kwargs['deepcopy']
    else:
        deepcopy = 0
    if(deepcopy):
        sdict = copy.deepcopy(sdict)
    else:
        pass
    if(clear):
        for i in range(sdict.__len__()-1,-1,-1):
            r = sdict[i]
            for j in range(r.__len__()-1,-1,-1):
                d = r[j]
                d['html_tag_lines'] = 0
                d['start_tagn'] = 0
                d['end_tagn'] = 0
                if(d['leaf']):
                    d['html_tag_lines'] = 3
                else:
                    d['html_tag_lines'] = 2
    else:
        pass
    for i in range(sdict.__len__()-1,-1,-1):
        r = sdict[i]
        for j in range(r.__len__()-1,-1,-1):
            d = r[j]
            pbp = copy.copy(d['breadth_path'])
            pbp.pop(-1)
            h,v = breadth_path_to_sdict_location(pbp)
            if((h==-1)|(v==-1)):
                pass
            else:
                sdict[h][v]['html_tag_lines'] = sdict[h][v]['html_tag_lines']+d['html_tag_lines']
    for i in range(0,sdict.__len__()):
        r = sdict[i]
        for j in range(0,r.__len__()):
            d = r[j]
            if((j==0)&(i==0)):
                cursor = 0
            ssp = copy.copy(d['siblings_seq_path'])
            sibn = ssp.pop(-1)
            if(sibn == 0):
                pbp = copy.copy(d['breadth_path'])
                pbp.pop(-1)
                h,v = breadth_path_to_sdict_location(pbp)
                if((h==-1)&(v==-1)):
                    pass
                else:
                    p = sdict[h][v]
                    cursor = p['start_tagn'] + 1
            else:
                pass
            d['start_tagn'] = cursor
            d['end_tagn'] = cursor + d['html_tag_lines'] -1
            cursor = d['end_tagn'] + 1
    html_lines = {}
    for i in range(0,sdict.__len__()):
        r = sdict[i]
        for j in range(0,r.__len__()):
            d = r[j]
            pol = copy.copy(d['orig_obj_path'])
            tag = pol.pop(-1)
            prepend = xml_indent_prepend(pol)
            if(noattrib):
                attrib =''
            else:
                attrib = attrib_dict_to_str(d['attrib'])
            html_lines[d['start_tagn']] = ''.join((prepend,'<',str(tag),attrib,'>'))
            html_lines[d['end_tagn']] = ''.join((prepend,'</',str(tag),'>'))
            #
            d['start_html_line'] = html_lines[d['start_tagn']]
            d['end_html_line'] = html_lines[d['end_tagn']]
            #
            if(d['leaf']):
                ol = copy.copy(d['orig_obj_path'])
                prepend = xml_indent_prepend(ol)
                #
                if(None == d['text']):
                    html_text = ''
                    prepend = ''
                else:
                    html_text = cgi.escape(str(d['text']))
                #
                html_lines[d['start_tagn']+1] = ''.join((prepend,html_text))
                #
                d['text_html_line'] = html_lines[d['start_tagn']+1]
                #
            else:
                #
                d['text_html_line'] = None
                #
    return({'sdict':sdict,'html_lines':html_lines})

def hdict_to_xml(hdict,**kwargs):
    if('disable_type' in kwargs):
        disable_type = kwargs['disable_type']
    else:
        disable_type = 0
    if('line_sp' in kwargs):
        line_sp = kwargs['line_sp']
    else:
        line_sp = '\n'
    temp = get_sdict_and_prdict_from_hdict(hdict,disable_type = disable_type)
    sdict = temp['sdict']
    prdict = temp['prdict']
    tmp = creat_xml_tag_line_label(sdict)
    sdict = tmp['sdict']
    tag_lines = tmp['html_lines']
    rslt = ''
    for i in range(0,tag_lines.__len__()):
        if(tag_lines[i] == ''):
            pass
        else:
            rslt = ''.join((rslt,tag_lines[i],line_sp))
    rslt = utils.str_rstrip(rslt,line_sp,1)
    return(rslt)


def hspr_to_xml(hspr,**kwargs):
    if('line_sp' in kwargs):
        line_sp = kwargs['line_sp']
    else:
        line_sp = '\n'
    sdict = hspr['sdict']
    prdict = hspr['prdict']
    tmp = creat_xml_tag_line_label(sdict)
    sdict = tmp['sdict']
    tag_lines = tmp['html_lines']
    rslt = ''
    for i in range(0,tag_lines.__len__()):
        if(tag_lines[i] == ''):
            pass
        else:
            rslt = ''.join((rslt,tag_lines[i],line_sp))
    rslt = utils.str_rstrip(rslt,line_sp,1)
    return(rslt)





def obj_to_xml(obj,**kwargs):
    if('line_sp' in kwargs):
        line_sp = kwargs['line_sp']
    else:
        line_sp = '\n'
    if('n2s' in kwargs):
        n2s = kwargs['n2s']
    else:
        n2s = 0
    if('s2n' in kwargs):
        s2n = kwargs['n2s']
    else:
        s2n = 0
    temp = obj_to_hdict(obj,s2n=s2n,n2s=n2s)
    sdict = temp['sdict']
    hdict = temp['hdict']
    prdict = temp['prdict']
    if('disable_type' in kwargs):
        disable_type = kwargs['disable_type']
    else:
        disable_type = 0
    rslt = hdict_to_xml(hdict,line_sp =line_sp,disable_type = disable_type)
    return(rslt)

def path_list_to_console_key_str(pl):
    rslt = ''
    for i in range(0,pl.__len__()):
        temp = pl[i]
        if(utils.is_int(temp)):
            temp = str(temp)
        else:
            temp = ''.join(('"',temp,'"'))
        rslt = ''.join((rslt,'[',temp,']'))
    return(rslt)
    
